@@ -1,21 +1,26 @@
 "bugs" <-
 function(data, inits, parameters.to.save, model.file="model.bug",
     n.chains=3, n.iter=2000, n.burnin=floor(n.iter / 2),
-    n.thin=max(1, floor(n.chains * (n.iter - n.burnin) / 1000)),
+    n.thin=max(1, floor(n.chains * (n.iter - n.burnin) / n.sims)), n.sims = 1000,
     bin=(n.iter - n.burnin) / n.thin,
     debug=FALSE, DIC=TRUE, digits=5, codaPkg=FALSE,
     bugs.directory="c:/Program Files/WinBUGS14/",
     program=c("WinBUGS", "OpenBUGS", "winbugs", "openbugs"),
     working.directory=NULL,
     clearWD=FALSE, useWINE=.Platform$OS.type != "windows", WINE=NULL,
-    newWINE=TRUE, WINEPATH=NULL)
+    newWINE=TRUE, WINEPATH=NULL, bugs.seed=NULL, summary.only=FALSE,
+    save.history=!summary.only)
 {
   program <- match.arg(program)
+  if (missing(bugs.directory) && 
+        !is.null(bugs.dir <- getOption("R2WinBUGS.bugs.directory"))) { # requested by Jouni Kerman
+      bugs.directory <- bugs.dir
+  }
   if(program %in% c("openbugs", "OpenBUGS", "OpenBugs")) {
     if(!is.R()) stop("OpenBUGS is not yet available in S-PLUS")
     ## If OpenBUGS, we only call openbugs() and exit...
     return(openbugs(data, inits, parameters.to.save, model.file,
-                    n.chains, n.iter, n.burnin, n.thin, DIC=DIC,
+                    n.chains, n.iter, n.burnin, n.thin, n.sims, DIC=DIC,
                     bugs.directory, working.directory, digits))
   }
   ## Checking number of inits, which is NOT save here:
@@ -36,16 +41,48 @@ function(data, inits, parameters.to.save, model.file="model.bug",
     setwd(working.directory)
     on.exit(setwd(savedWD))
   }
+  if(is.function(model.file)){
+      temp <- tempfile("model")
+      temp <-
+        if(is.R() || .Platform$OS.type != "windows"){
+               paste(temp, "txt", sep=".")
+        } else {
+               gsub("\\.tmp$", ".txt", temp)
+        }
+      write.model(model.file, con=temp)
+      model.file <- gsub("\\\\", "/", temp)
+      if(!is.R()) on.exit(file.remove(model.file), add=TRUE)
+  }
   if(!file.exists(model.file))
     stop(paste(model.file, "does not exist."))
   if(file.info(model.file)$isdir)
     stop(paste(model.file, "is a directory, but a file is required."))
-  if(!(length(data) == 1 && is.vector(data) && is.character(data) && data == "data.txt")) {
-    bugs.data(data, dir=getwd(), digits)
+  if (!(length(data) == 1 && is.vector(data) && is.character(data) && 
+       (regexpr("\\.txt$", data) > 0))) {
+    bugs.data.file <- bugs.data(data, dir = getwd(), digits)
   } else {
-    if(!file.exists(data)) stop("File data.txt does not exist.")
+    if(!file.exists(data))
+        stop("File", data, "does not exist.")
+    bugs.data.file <- data
   }
-  bugs.inits(inits, n.chains, digits)
+
+
+  if (is.character(inits)) {
+    if (!all(file.exists(inits))) {
+        stop("One or more inits files are missing")
+    }
+    if (length(inits)!=n.chains) {
+        stop("Need one inits file for each chain")
+    }
+    bugs.inits.files <- inits
+  } else {
+    if (!is.function(inits) && !is.null(inits) &&  (length(inits) != n.chains)) {
+        stop("Number of initialized chains (length(inits)) != n.chains")
+    }
+    bugs.inits.files <- bugs.inits(inits, n.chains, digits)
+  }
+
+
   if(DIC) parameters.to.save <- c(parameters.to.save, "deviance")
   ## Model files with extension ".bug" need to be renamed to ".txt"
   if(length(grep("[.]bug$", model.file))) {
@@ -58,17 +95,24 @@ function(data, inits, parameters.to.save, model.file="model.bug",
   bugs.script(parameters.to.save, n.chains, n.iter, n.burnin, n.thin,
               new.model.file, debug=debug, is.inits=!is.null(inits),
               bin=bin, DIC=DIC, useWINE=useWINE, newWINE=newWINE,
-              WINEPATH=WINEPATH)
+              WINEPATH=WINEPATH, bugs.seed=bugs.seed, 
+              summary.only=summary.only, save.history=save.history, 
+              bugs.data.file = bugs.data.file, 
+              bugs.inits.files = bugs.inits.files)
   bugs.run(n.burnin, bugs.directory, WINE=WINE, useWINE=useWINE,
            newWINE=newWINE, WINEPATH=WINEPATH)
   if(codaPkg)
     return(file.path(getwd(), paste("coda", 1:n.chains, ".txt", sep="")))
+  if (summary.only) {
+    return(bugs.log("log.txt"))
+  }
+
   sims <- c(bugs.sims(parameters.to.save, n.chains, n.iter, n.burnin,
                       n.thin, DIC),
             model.file=model.file, program=program)
   if(clearWD) {
-    file.remove(c("data.txt", "log.odc", "log.txt", "codaIndex.txt",
-                  paste("inits", 1:n.chains, ".txt", sep=""),
+    file.remove(c(bugs.data.file, "log.odc", "log.txt", "codaIndex.txt",
+                  bugs.inits.files,
                   paste("coda", 1:n.chains, ".txt", sep="")))
   }
   class(sims) <- "bugs"
